#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_catalog.interfaces import MINUTE_RESOLUTION, DATE_RESOLUTION
from pyams_content.interfaces import IBaseContent
from pyams_content.component.links.interfaces import IInternalLink, IInternalReferencesList
from pyams_content.root.interfaces import ISiteRootToolsConfiguration
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_utils.interfaces.site import ISiteGenerations
from pyams_utils.interfaces.traversing import IPathElements
from pyams_workflow.interfaces import IWorkflowState, IWorkflowPublicationInfo
from zope.dublincore.interfaces import IZopeDublinCore
from zope.site.interfaces import INewLocalSite

# import packages
from hypatia.text.lexicon import Lexicon
from pyams_catalog.index import FieldIndexWithInterface, KeywordIndexWithInterface, DatetimeIndexWithInterface
from pyams_catalog.nltk import NltkFullTextProcessor
from pyams_catalog.site import check_required_indexes
from pyams_content.shared.common.manager import SharedToolContainer
from pyams_content.shared.form.manager import FormsManager
from pyams_content.shared.imagemap.manager import ImageMapsManager
from pyams_content.shared.news.manager import NewsManager
from pyams_content.shared.view.manager import ViewsManager
from pyams_i18n.index import I18nTextIndexWithInterface
from pyams_security.index import PrincipalsRoleIndex
from pyams_utils.registry import utility_config, get_global_registry
from pyams_utils.site import check_required_utilities
from pyramid.events import subscriber
from pyramid.path import DottedNameResolver
from zope.lifecycleevent import ObjectCreatedEvent


def get_fulltext_lexicon(language):
    return Lexicon(NltkFullTextProcessor(language=language))


REQUIRED_UTILITIES = ()


REQUIRED_TOOLS = [('views', ViewsManager),
                  ('news', NewsManager),
                  ('forms', FormsManager),
                  ('imagemaps', ImageMapsManager)]


REQUIRED_INDEXES = [('content_type', FieldIndexWithInterface, {'interface': IBaseContent,
                                                               'discriminator': 'content_type'}),
                    ('role:owner', PrincipalsRoleIndex, {'role_id': 'pyams.Owner'}),
                    ('role:pilot', PrincipalsRoleIndex, {'role_id': 'pyams.Pilot'}),
                    ('role:manager', PrincipalsRoleIndex, {'role_id': 'pyams.Manager'}),
                    ('role:contributor', PrincipalsRoleIndex, {'role_id': 'pyams.Contributor'}),
                    ('role:webmaster', PrincipalsRoleIndex, {'role_id': 'pyams.Webmaster'}),
                    ('parents', KeywordIndexWithInterface, {'interface': IPathElements,
                                                            'discriminator': 'parents'}),
                    ('workflow_state', FieldIndexWithInterface, {'interface': IWorkflowState,
                                                                 'discriminator': 'state'}),
                    ('workflow_principal', FieldIndexWithInterface, {'interface': IWorkflowState,
                                                                     'discriminator': 'state_principal'}),
                    ('modifiers', KeywordIndexWithInterface, {'interface': IWfSharedContent,
                                                              'discriminator': 'modifiers'}),
                    ('created_date', DatetimeIndexWithInterface, {'interface': IZopeDublinCore,
                                                                  'discriminator': 'created',
                                                                  'resolution': DATE_RESOLUTION}),
                    ('modified_date', DatetimeIndexWithInterface, {'interface': IZopeDublinCore,
                                                                   'discriminator': 'modified',
                                                                   'resolution': DATE_RESOLUTION}),
                    ('publication_date', DatetimeIndexWithInterface, {'interface': IWorkflowPublicationInfo,
                                                                      'discriminator': 'publication_date',
                                                                      'resolution': MINUTE_RESOLUTION}),
                    ('effective_date', DatetimeIndexWithInterface, {'interface': IWorkflowPublicationInfo,
                                                                    'discriminator': 'publication_effective_date',
                                                                    'resolution': MINUTE_RESOLUTION}),
                    ('push_end_date', DatetimeIndexWithInterface, {'interface': IWorkflowPublicationInfo,
                                                                   'discriminator': 'push_end_date_index',
                                                                   'resolution': MINUTE_RESOLUTION}),
                    ('expiration_date', DatetimeIndexWithInterface, {'interface': IWorkflowPublicationInfo,
                                                                     'discriminator': 'publication_expiration_date',
                                                                     'resolution': MINUTE_RESOLUTION}),
                    ('first_publication_date', DatetimeIndexWithInterface, {'interface': IWorkflowPublicationInfo,
                                                                            'discriminator': 'first_publication_date',
                                                                            'resolution': MINUTE_RESOLUTION}),
                    ('link_reference', FieldIndexWithInterface, {'interface': IInternalLink,
                                                                 'discriminator': 'reference'}),
                    ('link_references', KeywordIndexWithInterface, {'interface': IInternalReferencesList,
                                                                    'discriminator': 'references'})]


def get_tools_manager(site, config, registry=None):
    """Check for shared tools manager"""
    if registry is None:
        registry = get_global_registry()
    name = config.tools_name or \
           registry.settings.get('pyams_content.config.tools_name', 'tools')
    if name not in site:
        manager = SharedToolContainer()
        registry.notify(ObjectCreatedEvent(manager))
        manager.title = {'en': "Shared tools",
                         'fr': "Outils partagés"}
        manager.short_name = {'en': "Shared tools",
                              'fr': "Outils partagés"}
        manager.navigation_name = {'en': "Shared tools",
                                   'fr': "Outils partagés"}
        site[name] = manager
        config.tools_name = name
    else:
        manager = site[name]
    return manager


def get_required_tools(manager, config, tools=REQUIRED_TOOLS, registry=None):
    """Create required shared tools"""
    if registry is None:
        registry = get_global_registry()
    for name, default_factory in tools:
        factory = registry.settings.get('pyams_content.config.{name}_tool_factory'.format(name=name))
        if (factory is None) or (factory.upper() not in ('NONE', '--')):
            attr_name = '{name}_tool_name'.format(name=name)
            tool_name = getattr(config, attr_name) or \
                        registry.settings.get('pyams_content.config.{name}'.format(name=attr_name), name)
            if tool_name not in manager:
                if factory is not None:
                    factory = DottedNameResolver().resolve(factory)
                else:
                    factory = default_factory
                tool = factory()
                registry.notify(ObjectCreatedEvent(tool))
                manager[tool_name] = tool
                setattr(config, attr_name, tool_name)


def check_required_tools(site, config_interface, tools):
    """Check for required shared tools"""
    configuration = config_interface(site, None)
    if configuration is not None:
        # check shared tools manager
        manager = get_tools_manager(site, configuration)
        # check for shared tools
        get_required_tools(manager, configuration, tools)


def get_required_indexes():
    indexes = REQUIRED_INDEXES
    registry = get_global_registry()
    for code, language in map(lambda x: x.split(':'),
                              registry.settings.get('pyams_content.lexicon.languages', 'en:english').split()):
        indexes.append(('title:{0}'.format(code), I18nTextIndexWithInterface,
                        {'language': code,
                         'interface': IBaseContent,
                         'discriminator': 'title',
                         'lexicon': lambda: get_fulltext_lexicon(language)}))
    return indexes


@subscriber(INewLocalSite)
def handle_new_local_site(event):
    """Check for required utilities when a site is created"""
    site = event.manager.__parent__
    check_required_utilities(site, REQUIRED_UTILITIES)
    check_required_indexes(site, get_required_indexes())
    check_required_tools(site, ISiteRootToolsConfiguration, REQUIRED_TOOLS)


@utility_config(name='PyAMS content', provides=ISiteGenerations)
class WebsiteGenerationsChecker(object):
    """PyAMS content package generations checker"""

    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_indexes(site, get_required_indexes())
        check_required_tools(site, ISiteRootToolsConfiguration, REQUIRED_TOOLS)
